;**************************************************************************
;                                 TEMPLATE
;**************************************************************************
 list       F=inhx8m, P=16F84A, R=hex, N=0
 __config   0x3FF9    ; Configuration Word (Fuses)
 errorlevel -302      ; switch off Message [302]:
                      ; Register in operand not in bank 0.

; Special Function Register names
indf    equ     0x00    ; bank0
tmr0    equ     0x01
pcl     equ     0x02
status  equ     0x03
fsr     equ     0x04
porta   equ     0x05
portb   equ     0x06
eedata  equ     0x08
eeadr   equ     0x09
pclath  equ     0x0A
intcon  equ     0x0B

optireg equ     0x81    ; bank1
trisa   equ     0x85
trisb   equ     0x86
eecon1  equ     0x88
eecon2  equ     0x89

; User Register definitions
wbak    equ     0x4E
statbak equ     0x4F

; User Register definitions
w       equ     0       ; instruction result to W, register unchanged
f       equ     1       ; instruction result to register, W unchanged

; Definitions of Bits
#define carry   status,0 ; STATUS register
#define dcarry  status,1
#define zero    status,2

; Macros
Bank0   macro
        bcf     status,5        ; BANK0 (Data Memory)
        endm

Bank1   macro
        bsf     status,5        ; BANK1 (Data Memory)
        endm

; Program Memory Beginning ------------------------------------------------
        org     0
        goto    Start

; Interrupt Service Routine -----------------------------------------------
        org     4               ; ISR beginning
        movwf   wbak            ; save off current W register contents
        movf    status,w        ; move status register into W register
        movwf   statbak         ; save off contents of STATUS register

;       bcf     <...IF>         ; clear interrupt flag

  ; ... ISR code here ...

        movf    statbak,w       ; restore STATUS
        movwf   status
        swapf   wbak,f          ; restore W without influence of STATUS
        swapf   wbak,w
        retfie                  ; return from ISR

; Subroutines -------------------------------------------------------------
; Microcontroller initialization
Init    movlw   b'00000000'     ; PORTA setup
        movwf   porta
        movlw   b'00000000'     ; PORTB setup
        movwf   portb
        Bank1
        movlw   b'11111111'     ; TRISA setup
        movwf   trisa
        movlw   b'11111111'     ; TRISB setup
        movwf   trisb
        movlw   b'11111111'     ; OPTION_REG setup
        movwf   optireg
        Bank0

        movlw   0x0C            ; clearing RAM
        movwf   fsr             ; RAM beginning to FSR
RamClr  clrf    indf            ; clear RAM cell
        incf    fsr,f
        movlw   0x50            ; all cleared?
        xorwf   fsr,w
        btfss   zero
        goto    RamClr
        return

; Main Program ------------------------------------------------------------
Start   call    Init    ; microcontroller initialization

  ; ... other start-up code ...

Main    nop             ; main loop beginning

  ; ... main loop body ...

        goto    Main    ; main loop end

        end             ; end of program (required directive)